# Benchmarks

Current repository: https://gitlab.gpf-tech.ign.fr/oslandia/pg_featureserv_test_tools.git

## Docker

### Build image

```bash
docker build -t pgfs_bench:latest -f ci/benchmarks/Dockerfile .
```

Arguments :
* **IMAGE**: go image name
* **IMAGE_VERSION**: go image version

### Run container

From projet root directory:

```bash
docker run -it --rm \
       -v $(pwd)/reports:/reports \
       -v PATH_LATEST_PG_FEAT_SRC:/usr/src/pg_featureserv_app \
       -v $(pwd):/usr/src/pg_bench_app \
       -e BENCHMARKS_ITERATION=1000 \
       --name my-pgfs pgfs_bench:latest
```

Available env variables:
* **HTTP_PROXY**: http proxy. Default: empty
* **HTTPS_PROXY**: https proxy. Default: empty
* **DATABASE_URL**: db connection string. Default: `postgresql://postgres:postgres@localhost:5432/pg_featureserv`
* **PGFS_CACHE_REDIS_URL**: redis connection string. Default: `localhost:6379`
* **PGFS_CACHE_REDIS_PASSWORD**: redis password. Default: empty
* **BENCHMARKS_GO_DIR**: go path to find the benchmark scripts. Default: `github.com/CrunchyData/pg_featureserv/internal/benchmarks`
* **BENCHMARKS_REF**: build reference data to compare with. Default: `true`
* **BENCHMARKS_REF_REV**: **pg_featureserv** version used to build the reference data. Default: `v1.2.1`
* **BENCHMARKS_ITERATION**: number of benchmark iteration. Default: 10
* **BENCHMARKS_PATCH_DIR**: path inside the docker image where the patchs can be found. Default: `/usr/src/pg_bench_app/benchmarks`
* **BENCHMARKS_REVISIONS**: **pg_featureserv** versions to bench. Default: `v1.3.0 v1.3.1 v1.3.2 latest`
* **BENCHMARKS_PROFILE**: Create profiling data. Default: "false"
* **BENCHMARKS_REGEX**: Select benchmark to run. Default: empty ie. all
* **BENCHMARKS_REPORT_DIR**: output report directory. Default: /reports

## Docker compose

You can adjust the env variables as specified above.

### Build image

From projet root directory:

```bash
docker-compose -f benchmarks/docker-compose.yml build
```

### Run stack

```bash
docker-compose -f benchmarks/docker-compose.yml up
```

## Benchmark logic

### Docker compose

The main logic is to run a set of benchmark tests for different **pg_featureserv** versions and create a `benchstat` report against a referenced version (in our case `v1.2.1`).

The benchmark tests are the same for all **pg_featureserv** versions and to have these tests working with older **pg_featureserv** versions, a patch may be applied on **pg_featureserv**. The `entrypoint.sh` script looks for patch files in `BENCHMARKS_PATCH_DIR` matching the **pg_featureserv** version under test. Patch files have this format: `github-crunchydata-VERSION.patch` where `VERSION` is the **pg_featureserv** version.

Each **pg_featureserv** version is tested against the 3 caches available: **disabled**, **naive**, **redis**.

Reports are available in the projet root directory.

### Benchmarked tests

* **getCollections** :

`GET /collections/public.mock_a`

* **getCollectionItems** :

```text
GET /collections/public.mock_a/items
GET /collections/public.mock_b/items
GET /collections/public.mock_c/items
```

* **getCollectionItem** :

```text
GET /collections/public.mock_a/items/1
GET /collections/public.mock_a/items/1 avec passage de l'ETag
```
