package db_bench

/*
 Copyright 2022 Crunchy Data Solutions, Inc.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
      http://www.apache.org/licenses/LICENSE-2.0
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 Date     : Octobre 2022
 Authors  : Jean-philippe Bazonnais (jean-philippe dot bazonnais at ign dot fr)
*/

import (
	"net/http"
	"testing"

	"github.com/CrunchyData/pg_featureserv/internal/conf"
)

const path = "/collections/public.mock_a/items/1"

func BenchmarkGetCollectionItem(b *testing.B) {
	// TODO
	// on peut aussi tester la fonction handler pour avoir des metriques plus précis
	// cf. https://blog.questionable.services/article/testing-http-handlers-go/

	rr := hTest.DoRequest(b, path)
	etag := rr.Header().Get("ETag")

	expectedCode := 200
	if conf.Configuration.Cache.Type == "Redis" || conf.Configuration.Cache.Type == "Naive" {
		expectedCode = 304
	}

	b.ResetTimer()
	b.RunParallel(func(pb *testing.PB) {
		for pb.Next() {
			b.StartTimer()
			header := http.Header{}
			header.Add("If-None-Match", etag)
			hTest.DoRequestMethodStatus(b, "GET", path, nil, header, expectedCode)
			b.StopTimer()
		}
	})
}
